////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.miso;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.net.URL;
import java.util.Calendar;

import org.watto.android.Language;
import org.watto.android.R;
import org.watto.program.android.sync.miso.array.ArrayResizer;
import org.watto.program.android.sync.miso.datatype.CheckInData;
import org.watto.program.android.sync.miso.datatype.TaglineData;
import org.watto.program.android.sync.miso.datatype.StatusData;
import org.watto.program.android.sync.miso.datatype.UserIdData;
import org.watto.program.android.sync.miso.datatype.WebsiteData;
import android.content.ContentProviderOperation;
import android.content.ContentValues;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.provider.ContactsContract;
import android.provider.ContactsContract.CommonDataKinds.Email;
import android.provider.ContactsContract.CommonDataKinds.GroupMembership;
import android.provider.ContactsContract.CommonDataKinds.Im;
import android.provider.ContactsContract.CommonDataKinds.Nickname;
import android.provider.ContactsContract.CommonDataKinds.Phone;
import android.provider.ContactsContract.CommonDataKinds.Photo;
import android.provider.ContactsContract.CommonDataKinds.StructuredName;
import android.provider.ContactsContract.CommonDataKinds.StructuredPostal;
import android.provider.ContactsContract.Data;
import android.provider.ContactsContract.Groups;
import android.provider.ContactsContract.RawContacts;


/***********************************************************************************************
Helper class for storing data in the platform content providers
***********************************************************************************************/
public class ContactOperations {

  ContentValues content;
  ContentProviderOperation.Builder operationBuilder;
  BatchOperation batchOperation;
  Context context;
  boolean yield;
  long rawContactId = 0;
  int backReference;
  boolean newContact;


  /***********************************************************************************************
  Constructor
  ***********************************************************************************************/
  public ContactOperations(Context context, BatchOperation batchOperation){
    this.content = new ContentValues();
    this.yield = true;
    this.context = context;
    this.batchOperation = batchOperation;
  }


  /***********************************************************************************************
  Constructor for adding a new User
  ***********************************************************************************************/
  public ContactOperations(Context context, int userId, String accountName, BatchOperation batchOperation){
    this(context,batchOperation);
    this.backReference = batchOperation.size();
    this.newContact = true;

    content.put(RawContacts.SOURCE_ID,userId);
    content.put(RawContacts.ACCOUNT_TYPE,GlobalConstants.ACCOUNT_TYPE);
    content.put(RawContacts.ACCOUNT_NAME,accountName);

    this.operationBuilder = addInsertOperation(RawContacts.CONTENT_URI,true).withValues(content);
    batchOperation.add(operationBuilder.build());
  }


  /***********************************************************************************************
  Constructor for adding a new Group
  ***********************************************************************************************/
  public ContactOperations(Context context, int groupId, String accountName, BatchOperation batchOperation, boolean addingGroup){
    this(context,batchOperation);
    this.backReference = batchOperation.size();
    this.newContact = true;
  }


  /***********************************************************************************************
  Constructor for updating an existing User
  ***********************************************************************************************/
  public ContactOperations(Context context, long rawContactId, BatchOperation batchOperation){
    this(context,batchOperation);
    this.newContact = false;
    this.rawContactId = rawContactId;
  }


  /***********************************************************************************************
  Adds a checkin
  @param checkin the checkin
  @param checkinTime the checkin time
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addCheckin(String checkin,long checkinTime){
    content.clear();
    if (checkin != null && !checkin.equals("")) {
      content.put(Data.MIMETYPE,CheckInData.CONTENT_ITEM_TYPE);
      content.put(CheckInData.LABEL,context.getString(R.string.label_checkin));
      content.put(CheckInData.VENUE,checkin);
      content.put(CheckInData.TIMESTAMP,checkinTime);

      // Build the checkin message
      String checkinMessage = "";

      if (checkin != null && !checkin.equals("")) {
        checkinMessage += checkin;
      }

      if (checkinTime > 0) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(checkinTime);

        if (!checkinMessage.equals("")) {
          checkinMessage += " at " + formatTimestamp(calendar);
        }
      }

      if (!checkinMessage.equals("")) {
        content.put(CheckInData.DATA,checkinMessage);
      }

    }

    if (content.size() > 0) {
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds an update operation into the batch
  ***********************************************************************************************/
  public void addDeleteOperation(Uri uri){
    operationBuilder = addDeleteOperation(uri,yield);
    yield = false;
    batchOperation.add(operationBuilder.build());
  }
  
  
  /***********************************************************************************************
  Adds a tagline
  @param tagline the tagline to add
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addTagline(String tagline){
    content.clear();
    if (tagline != null && !tagline.equals("")) {
      content.put(TaglineData.DATA,tagline);
      content.put(TaglineData.LABEL,context.getString(R.string.label_tagline));
      content.put(Data.MIMETYPE,TaglineData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Updates the tagline
  @param tagline the new tagline
  @param oldTagline the old tagline
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateTagline(String tagline,String oldTagline,Uri contactUrl){
    if (tagline != null && oldTagline != null && !tagline.equals(oldTagline)) {
      content.clear();
      content.put(TaglineData.DATA,tagline);
      addUpdateOperation(contactUrl);
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Adds a website
  @param website the website to add
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addWebsite(String website){
    content.clear();
    if (website != null && !website.equals("")) {
      content.put(WebsiteData.DATA,website);
      content.put(WebsiteData.LABEL,context.getString(R.string.label_website));
      content.put(Data.MIMETYPE,WebsiteData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Updates the website
  @param website the new website
  @param oldWebsite the old website
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateWebsite(String website,String oldWebsite,Uri contactUrl){
    if (website != null && oldWebsite != null && !website.equals(oldWebsite)) {
      content.clear();
      content.put(WebsiteData.DATA,website);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Adds an email address
  @param new email for user
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addEmail(String email){
    content.clear();
    if (email != null && !email.equals("")) {
      content.put(Email.DATA,email);
      content.put(Email.TYPE,Email.TYPE_OTHER);
      content.put(Email.MIMETYPE,Email.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds a Facebook account
  @param facebook the facebook account
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addFacebook(String facebook){
    content.clear();
    if (facebook != null && !facebook.equals("")) {
      content.put(Im.DATA,facebook);
      content.put(Im.TYPE,Im.TYPE_HOME);
      content.put(Im.PROTOCOL,Im.PROTOCOL_CUSTOM);
      content.put(Im.CUSTOM_PROTOCOL,"Facebook");
      content.put(Im.MIMETYPE,Im.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }

  
  /***********************************************************************************************
  Updates the users real name
  @param realName the new name
  @param oldRealName the old name
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateRealName(String realName,String oldRealName,Uri contactUrl){
    content.clear();
    if (realName != null && oldRealName != null && !realName.equals(oldRealName)) {
      content.put(StructuredName.DISPLAY_NAME,realName);
    }
    if (content.size() > 0) {
      addUpdateOperation(contactUrl);
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Adds the username of a user
  @param username the username
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addUsername(String username){
    content.clear();
    if (username != null && !username.equals("")) {
      content.put(Nickname.NAME,username);
      content.put(Nickname.MIMETYPE,Nickname.CONTENT_ITEM_TYPE);
      content.put(Nickname.TYPE,Nickname.TYPE_CUSTOM);
      content.put(Nickname.LABEL,Language.get(R.string.label_nickname));
    }
    if (content.size() > 0) {
      addInsertOperation();
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Updates the users username
  @param username the new username
  @param oldUsername the old username
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateUsername(String username,String oldUsername,Uri contactUrl){
    content.clear();
    if (username != null && oldUsername != null && !username.equals(oldUsername)) {
      content.put(Nickname.NAME,username);
    }
    if (content.size() > 0) {
      addUpdateOperation(contactUrl);
    }
    return this;
  }
  
  
  /***********************************************************************************************
  Adds the real name of a user
  @param realName the real name of the user
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addRealName(String realName){
    content.clear();
    if (realName != null && !realName.equals("")) {
      content.put(StructuredName.DISPLAY_NAME,realName);
      content.put(StructuredName.MIMETYPE,StructuredName.CONTENT_ITEM_TYPE);
    }
    if (content.size() > 0) {
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds a "Miso" group
  @param title the name of the group
  @param notes a description of the group
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addGroup(String title,String notes){

    content.clear();
    if (title != null && !title.equals("") && notes != null && !notes.equals("")) {
      content.put(Groups.ACCOUNT_TYPE,GlobalConstants.ACCOUNT_TYPE);
      content.put(Groups.ACCOUNT_NAME,GlobalConstants.ACCOUNT_NAME);
      content.put(Groups.TITLE,title);
      content.put(Groups.NOTES,notes);
      content.put(Groups.GROUP_VISIBLE,"1");
      content.put(Groups.SHOULD_SYNC,"1");
    }

    if (content.size() > 0) {
      operationBuilder = addInsertOperation(addCallerIsSyncAdapterParameter(Groups.CONTENT_URI),yield);
      operationBuilder.withValues(content);

      yield = false;
      batchOperation.add(operationBuilder.build());
      // MUST execute here, so the group is created and can be referenced when adding users to the group
      batchOperation.execute();

    }

    return this;

  }


  /***********************************************************************************************
  Adds a contact to a group
  @param groupId the ID of the group to add the contact to
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addGroupMembership(long groupId){
    content.clear();
    if (groupId != -1) {
      content.put(Data.MIMETYPE,GroupMembership.CONTENT_ITEM_TYPE);
      content.put(GroupMembership.GROUP_ROW_ID,groupId);
    }

    if (content.size() > 0) {
      addInsertOperation();
    }
    return this;
  }





  /***********************************************************************************************
  Adds an insert operation into the batch
  ***********************************************************************************************/
  public void addInsertOperation(){

    if (!newContact) {
      content.put(Phone.RAW_CONTACT_ID,rawContactId);
    }

    operationBuilder = addInsertOperation(addCallerIsSyncAdapterParameter(Data.CONTENT_URI),yield);
    operationBuilder.withValues(content);

    if (newContact) {
      operationBuilder.withValueBackReference(Data.RAW_CONTACT_ID,backReference);
    }

    yield = false;
    batchOperation.add(operationBuilder.build());
  }


  /***********************************************************************************************
  Adds a contact name
  @param name Name of contact
  @param nameType type of name: family name, given name, etc.
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addName(String firstName,String lastName){
    content.clear();
    if (firstName != null && !firstName.equals("")) {
      content.put(StructuredName.GIVEN_NAME,firstName);
      content.put(StructuredName.MIMETYPE,StructuredName.CONTENT_ITEM_TYPE);
    }
    if (lastName != null && !lastName.equals("")) {
      content.put(StructuredName.FAMILY_NAME,lastName);
      content.put(StructuredName.MIMETYPE,StructuredName.CONTENT_ITEM_TYPE);
    }
    if (content.size() > 0) {
      addInsertOperation();
    }
    return this;
  }



  /***********************************************************************************************
  Adds a photo
  @param photoURL the URL to the photo
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addPhoto(URL photoURL){
    content.clear();

    if (photoURL != null && !photoURL.toString().equals("")) {
      // Download the photo into a byte[] BLOB (stored as a String)
      byte[] photoBytes = downloadPhoto(photoURL);

      // Only add a photo if the download was successful
      if (photoBytes != null && photoBytes.length > 0) {
        // store the date/time that this photo was retrieved, so its only grabbed once a day
        String timestamp = "" + Calendar.getInstance().getTimeInMillis();

        // Store the photo
        content.put(Photo.PHOTO,photoBytes);
        content.put(Photo.SYNC1,timestamp);
        content.put(Photo.MIMETYPE,Photo.CONTENT_ITEM_TYPE);
        addInsertOperation();
      }
    }
    return this;
  }


  /***********************************************************************************************
  Adds a "Miso" status message
  @param shoutout the users shoutout message
  @param checkin the last check-in location
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addStatus(String shoutout,String checkin){

    shoutout = buildShoutoutMessage(shoutout,checkin);

    content.put(StatusData.DATA,shoutout);
    content.put(StatusData.LABEL,context.getString(R.string.label_status));
    content.put(Data.MIMETYPE,StatusData.CONTENT_ITEM_TYPE);
    addInsertOperation();

    return this;

  }


  /***********************************************************************************************
  Adds a "Miso" status icon
  @param shoutout the users shoutout message
  @param checkin the last check-in location
  @param dataId the ID of the status data
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addStatusIcon(String shoutout,String checkin,long dataId){
    content.clear();

    shoutout = buildShoutoutMessage(shoutout,checkin);

    if (dataId != -1) {

      content.put(ContactsContract.StatusUpdates.DATA_ID,dataId);
      content.put(ContactsContract.StatusUpdates.STATUS,shoutout);
      content.put(ContactsContract.StatusUpdates.STATUS_RES_PACKAGE,GlobalConstants.ACCOUNT_TYPE);
      content.put(ContactsContract.StatusUpdates.STATUS_LABEL,context.getString(R.string.status_label));
      content.put(ContactsContract.StatusUpdates.STATUS_ICON,R.drawable.small_icon);
      content.put(ContactsContract.StatusUpdates.STATUS_TIMESTAMP,System.currentTimeMillis());
    }

    if (content.size() > 0) {
      // insert() does a replace() for status messages
      operationBuilder = addInsertOperation(addCallerIsSyncAdapterParameter(ContactsContract.StatusUpdates.CONTENT_URI),yield);
      operationBuilder.withValues(content);

      yield = false;
      batchOperation.add(operationBuilder.build());
      //batchOperation.execute();
    }

    return this;

  }


  /***********************************************************************************************
  Adds a Twitter account
  @param twitter the twitter account
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addTwitter(String twitter){
    content.clear();
    if (twitter != null && !twitter.equals("")) {
      content.put(Im.DATA,twitter);
      content.put(Im.TYPE,Im.TYPE_HOME);
      content.put(Im.PROTOCOL,Im.PROTOCOL_CUSTOM);
      content.put(Im.CUSTOM_PROTOCOL,"Twitter");
      content.put(Im.MIMETYPE,Im.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Adds an update operation into the batch
  ***********************************************************************************************/
  public void addUpdateOperation(Uri uri){
    operationBuilder = addUpdateOperation(uri,yield).withValues(content);
    yield = false;
    batchOperation.add(operationBuilder.build());
  }


  /***********************************************************************************************
  Adds the users backend server userId
  @param userID the userID on the backend server
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations addUserId(long userId){
    content.clear();
    if (userId >= 0) {
      content.put(UserIdData.DATA,userId);
      content.put(UserIdData.LABEL,context.getString(R.string.label_userId));
      content.put(Data.MIMETYPE,UserIdData.CONTENT_ITEM_TYPE);
      addInsertOperation();
    }
    return this;
  }


  /***********************************************************************************************
  Builds a shoutout message based on the <i>shoutout</i> set by the friend and the <i>checkin</i>
  @param shoutout the users shoutout message
  @param checkin the last check-in location
  @return the shoutout message
  ***********************************************************************************************/
  public String buildShoutoutMessage(String shoutout,String checkin){
    if (shoutout == null || shoutout.equals("")) {
      if (checkin != null && !checkin.equals("")) {
        // Show the message "Checked in at <location>"
        shoutout = context.getString(R.string.checked_in_at) + " " + checkin;
      }
      else {
        // Set the initial shoutout to be empty, so we can still do status and icon updates, etc.
        shoutout = context.getString(R.string.no_shoutout_message);
      }
    }
    else if (checkin != null && !checkin.equals("")) {
      shoutout = "'" + shoutout + "' " + context.getString(R.string.while_watching) + " " + checkin;
    }

    return shoutout;
  }


  /***********************************************************************************************
  Updates the users photo
  @param photoURL the new photo URL
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations deletePhoto(Uri contactUrl){
    addDeleteOperation(contactUrl);
    return this;
  }


  /***********************************************************************************************
  Downloads a photo from a <code>URL</code>. If the image is too large, it is resized.
  @param photoURL the URL to the photo
  @return the <code>byte</code> array of photo data (in JPEG format)
  ***********************************************************************************************/
  public byte[] downloadPhoto(URL photoURL){

    if (photoURL != null && !photoURL.toString().equals("")) {
      // Download the photo into a byte[] BLOB (stored as a String)
      byte[] photoBytes = new byte[1000];
      boolean photoSuccess = false;
      try {
        InputStream dataStream = photoURL.openStream();

        int readSize = 0;
        int receivedSize = dataStream.read(photoBytes,readSize,1000);

        while (receivedSize > 0) {
          // there is more data, so increase the array by another 1000 bytes and repeat the read
          readSize += receivedSize;
          photoBytes = ArrayResizer.resize(photoBytes,readSize + 1000);
          receivedSize = dataStream.read(photoBytes,readSize,1000);
        }

        dataStream.close();

        // resize to the actual size of the content
        photoBytes = ArrayResizer.resize(photoBytes,readSize);

        // Now check that the bytes are a valid JPEG file
        // (in case the URL was invalid or a 404 page was downloaded, etc)
        // (ie. only store the image if it is actually an image)
        Bitmap loadedJpg = BitmapFactory.decodeByteArray(photoBytes,0,readSize);
        if (loadedJpg != null) {
          // if the bitmap is larger than 150x150, resize it...
          int width = loadedJpg.getWidth();
          int height = loadedJpg.getHeight();
          if (width > 150 || height > 150) {
            width = 150;
            height = 150;
            // resize it
            loadedJpg = Bitmap.createScaledBitmap(loadedJpg,width,height,true);
            // get the new bytes for the resized image
            ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
            loadedJpg.compress(Bitmap.CompressFormat.JPEG,75,byteStream);
            photoBytes = byteStream.toByteArray();
          }
          photoSuccess = true;
        }
      }
      catch (Throwable t) {
        // Error downloading the photo
      }

      // Only add a photo if the download was successful
      if (photoSuccess) {
        return photoBytes;
      }

    }

    return null;
  }


  /***********************************************************************************************
  Formats a timestamp in a <code>Calendar</code> into a text <code>String</code>
  @param calendar the <code>Calendar</code> set to the timestamp
  @return the <code>String</code> representation of the timestamp
  ***********************************************************************************************/
  public String formatTimestamp(Calendar calendar){

    // Hour
    String timestamp = calendar.get(Calendar.HOUR) + ":";

    // Minute
    int minute = calendar.get(Calendar.MINUTE);
    if (minute < 10) {
      timestamp += "0";
    }
    timestamp += minute;

    // AM/PM
    if (calendar.get(Calendar.HOUR_OF_DAY) > 12) {
      timestamp += context.getString(R.string.time_pm) + " ";
    }
    else {
      timestamp += context.getString(R.string.time_am) + " ";
    }

    // Day
    int day = calendar.get(Calendar.DAY_OF_WEEK);
    if (day == Calendar.MONDAY) {
      timestamp += context.getString(R.string.day_monday) + " ";
    }
    else if (day == Calendar.TUESDAY) {
      timestamp += context.getString(R.string.day_tuesday) + " ";
    }
    else if (day == Calendar.WEDNESDAY) {
      timestamp += context.getString(R.string.day_wednesday) + " ";
    }
    else if (day == Calendar.THURSDAY) {
      timestamp += context.getString(R.string.day_thursday) + " ";
    }
    else if (day == Calendar.FRIDAY) {
      timestamp += context.getString(R.string.day_friday) + " ";
    }
    else if (day == Calendar.SATURDAY) {
      timestamp += context.getString(R.string.day_saturday) + " ";
    }
    else if (day == Calendar.SUNDAY) {
      timestamp += context.getString(R.string.day_sunday) + " ";
    }

    // Day number
    day = calendar.get(Calendar.DAY_OF_MONTH);
    timestamp += day;

    if (day == 1 || day == 21 || day == 31) {
      timestamp += context.getString(R.string.day_suffix_st) + " ";
    }
    else if (day == 2 || day == 22) {
      timestamp += context.getString(R.string.day_suffix_nd) + " ";
    }
    else if (day == 3 || day == 23) {
      timestamp += context.getString(R.string.day_suffix_rd) + " ";
    }
    else {
      timestamp += context.getString(R.string.day_suffix_th) + " ";
    }

    // Month
    int month = calendar.get(Calendar.MONTH);
    if (month == Calendar.JANUARY) {
      timestamp += context.getString(R.string.month_january) + " ";
    }
    else if (month == Calendar.FEBRUARY) {
      timestamp += context.getString(R.string.month_february) + " ";
    }
    else if (month == Calendar.MARCH) {
      timestamp += context.getString(R.string.month_march) + " ";
    }
    else if (month == Calendar.APRIL) {
      timestamp += context.getString(R.string.month_april) + " ";
    }
    else if (month == Calendar.MAY) {
      timestamp += context.getString(R.string.month_may) + " ";
    }
    else if (month == Calendar.JUNE) {
      timestamp += context.getString(R.string.month_june) + " ";
    }
    else if (month == Calendar.JULY) {
      timestamp += context.getString(R.string.month_july) + " ";
    }
    else if (month == Calendar.AUGUST) {
      timestamp += context.getString(R.string.month_august) + " ";
    }
    else if (month == Calendar.SEPTEMBER) {
      timestamp += context.getString(R.string.month_september) + " ";
    }
    else if (month == Calendar.OCTOBER) {
      timestamp += context.getString(R.string.month_october) + " ";
    }
    else if (month == Calendar.NOVEMBER) {
      timestamp += context.getString(R.string.month_november) + " ";
    }
    else if (month == Calendar.DECEMBER) {
      timestamp += context.getString(R.string.month_december) + " ";
    }

    // Year
    timestamp += calendar.get(Calendar.YEAR);

    return timestamp;
  }


  /***********************************************************************************************
  Updates the checkin
  @param checkin the new checkin
  @param oldCheckin the old checkin
  @param checkinTime the checkin time
  @param oldCheckinTime the old checkin time
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateCheckin(String checkin,String oldCheckin,long checkinTime,long oldCheckinTime,Uri contactUrl){
    content.clear();

    String checkinMessage = "";

    if (oldCheckin == null || (checkin != null && !checkin.equals(oldCheckin))) {
      content.put(CheckInData.VENUE,checkin);
      checkinMessage += checkin;
    }

    if (checkinTime != oldCheckinTime || !checkinMessage.equals("")) {
      // Force the timestamp to be set if the checkinMessage has been set
      content.put(CheckInData.TIMESTAMP,checkinTime);

      Calendar calendar = Calendar.getInstance();
      calendar.setTimeInMillis(checkinTime);

      if (checkinMessage.equals("")) {
        // The timestamp has changed, so we need to force the updating of the check-in location
        content.put(CheckInData.VENUE,checkin);
        checkinMessage += checkin;
      }
      checkinMessage += " at " + formatTimestamp(calendar);
    }

    if (!checkinMessage.equals("")) {
      content.put(CheckInData.DATA,checkinMessage);
    }

    if (content.size() > 0) {
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates contact's email
  @param email the new email address
  @param oldEmail the old email address
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateEmail(String email,String oldEmail,Uri contactUrl){
    if (email != null && oldEmail != null && !email.equals(oldEmail)) {
      content.clear();
      content.put(Email.DATA,email);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the users Facebook account
  @param facebook the new Facebook account
  @param oldFacebook the old Facebook account
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateFacebook(String facebook,String oldFacebook,Uri contactUrl){
    if (facebook != null && oldFacebook != null && !facebook.equals(oldFacebook)) {
      content.clear();
      content.put(Im.DATA,facebook);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the users gender
  @param gender the new gender
  @param oldGender the old gender
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateGender(String gender,String oldGender,Uri contactUrl){
    if (gender != null && oldGender != null && !gender.equals(oldGender)) {
      content.clear();
      content.put(TaglineData.DATA,gender);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the home city
  @param homeCity the new home city
  @param oldHomeCity the old home city
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateHomeCity(String homeCity,String oldHomeCity,Uri contactUrl){
    content.clear();
    if (homeCity != null && oldHomeCity != null && !homeCity.equals(oldHomeCity)) {
      content.put(StructuredPostal.FORMATTED_ADDRESS,homeCity);

      int commaPosition = homeCity.indexOf(',');
      if (commaPosition > 0) {
        String city = homeCity.substring(0,commaPosition);
        content.put(StructuredPostal.CITY,city);
        String country = homeCity.substring(commaPosition + 1).trim();
        content.put(StructuredPostal.COUNTRY,country);
      }

    }

    if (content.size() > 0) {
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates contact's name
  @param firstName the new first name
  @param oldFirstName the old first name
  @param lastName the new last name
  @param oldLastName the old last name
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateName(String firstName,String oldFirstName,String lastName,String oldLastName,Uri contactUrl){
    content.clear();
    if (firstName != null && oldFirstName != null && !firstName.equals(oldFirstName)) {
      content.put(StructuredName.GIVEN_NAME,firstName);
    }
    if (lastName != null && oldLastName != null && !lastName.equals(oldLastName)) {
      content.put(StructuredName.FAMILY_NAME,lastName);
    }
    if (content.size() > 0) {
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates contact's phone
  @param phone the new phone number
  @param oldPhone the old phone number
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updatePhone(String phone,String oldPhone,Uri contactUrl){
    if (phone != null && oldPhone != null && !phone.equals(oldPhone)) {
      content.clear();
      content.put(Phone.NUMBER,phone);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the users photo
  @param photoURL the new photo URL
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updatePhoto(URL photoURL,Uri contactUrl){
    if (photoURL != null && !photoURL.toString().equals("")) {
      content.clear();

      // Download the photo into a byte[] BLOB (stored as a String)
      byte[] photoBytes = downloadPhoto(photoURL);

      // Only update the photo if the download was successful
      if (photoBytes != null && photoBytes.length > 0) {
        // Store the photo
        content.put(Photo.PHOTO,photoBytes);
        addUpdateOperation(contactUrl);
      }

    }
    return this;
  }


  /***********************************************************************************************
  Updates a "Miso" status message
  @param shoutout the users shoutout message
  @param oldShoutout the old shoutout
  @param checkin the last check-in location
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateStatus(String shoutout,String oldShoutout,String checkin,Uri contactUrl){

    shoutout = buildShoutoutMessage(shoutout,checkin);
    
    if (shoutout == null || oldShoutout == null || (shoutout != null && oldShoutout != null && !shoutout.equals(oldShoutout))) {
      content.put(StatusData.DATA,shoutout);
      addUpdateOperation(contactUrl);
    }

    return this;

  }


  /***********************************************************************************************
  Updates the "Miso" status icon
  @param shoutout the new shoutout
  @param oldShoutout the old shoutout
  @param checkin the last check-in location
  @param oldCheckin the old check-in location
  @param dataId the ID of the status data
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateStatusIcon(String shoutout,String oldShoutout,String checkin,long dataId){
    String newShoutout = buildShoutoutMessage(shoutout,checkin);
    if (newShoutout == null || oldShoutout == null || (newShoutout != null && oldShoutout != null && !newShoutout.equals(oldShoutout))) {
      return addStatusIcon(shoutout,checkin,dataId);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the users Twitter account
  @param twitter the new Twitter account
  @param oldTwitter the old Twitter account
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateTwitter(String twitter,String oldTwitter,Uri contactUrl){
    if (twitter != null && oldTwitter != null && !twitter.equals(oldTwitter)) {
      content.clear();
      content.put(Im.DATA,twitter);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Updates the users backend server user ID
  @param userId the new user ID
  @param oldUserId the old user ID
  @param contactUrl the Uri for the existing raw contact to be updated
  @return instance of ContactOperations
  ***********************************************************************************************/
  public ContactOperations updateUserId(long userId,long oldUserId,Uri contactUrl){
    if (userId >= 0 && oldUserId >= 0 && userId != oldUserId) {
      content.clear();
      content.put(UserIdData.DATA,userId);
      addUpdateOperation(contactUrl);
    }
    return this;
  }


  /***********************************************************************************************
  Adds a CALLER_IS_SYNCADAPTER parameter
  ***********************************************************************************************/
  public static Uri addCallerIsSyncAdapterParameter(Uri uri){
    return uri.buildUpon().appendQueryParameter(ContactsContract.CALLER_IS_SYNCADAPTER,"true").build();
  }


  /***********************************************************************************************
  Builds a <code>ContentProviderOperation</code> for a delete operation
  ***********************************************************************************************/
  public static ContentProviderOperation.Builder addDeleteOperation(Uri uri,boolean yield){
    return ContentProviderOperation.newDelete(addCallerIsSyncAdapterParameter(uri)).withYieldAllowed(yield);

  }


  /***********************************************************************************************
  Builds a <code>ContentProviderOperation</code> for an insert operation
   ***********************************************************************************************/
  public static ContentProviderOperation.Builder addInsertOperation(Uri uri,boolean yield){
    return ContentProviderOperation.newInsert(addCallerIsSyncAdapterParameter(uri)).withYieldAllowed(yield);
  }


  /***********************************************************************************************
  Builds a <code>ContentProviderOperation</code> for an update operation
   ***********************************************************************************************/
  public static ContentProviderOperation.Builder addUpdateOperation(Uri uri,boolean yield){
    return ContentProviderOperation.newUpdate(addCallerIsSyncAdapterParameter(uri)).withYieldAllowed(yield);
  }


  /***********************************************************************************************
  Creates a new <code>ContactOperations</code> for adding new contact to the Android Contacts
  @param context the Authenticator Activity context
  @param userId the userId of the sample SyncAdapter user object
  @param accountName the username of the current login
  @param batchOperations any batch operations to perform when creating the new contact 
  @return instance of ContactOperations
  ***********************************************************************************************/
  public static ContactOperations createNewContact(Context context,int userId,String accountName,BatchOperation batchOperations){
    return new ContactOperations(context,userId,accountName,batchOperations);
  }


  /***********************************************************************************************
  Creates a new <code>ContactOperations</code> for adding new group to the Android Contacts
  @param context the Authenticator Activity context
  @param groupId the groupId of the sample SyncAdapter user object
  @param accountName the username of the current login
  @param batchOperations any batch operations to perform when creating the new contact 
  @return instance of ContactOperations
  ***********************************************************************************************/
  public static ContactOperations createNewGroup(Context context,int groupId,String accountName,BatchOperation batchOperations){
    return new ContactOperations(context,groupId,accountName,batchOperations,true);
  }


  /***********************************************************************************************
  Creates a new <code>ContactOperations</code> for updating an existing Android Contact
  @param context the Authenticator Activity context
  @param rawContactId the unique ID of the existing contact
  @param batchOperations any batch operations to perform when updating the contact 
  @return instance of ContactOperations
   ***********************************************************************************************/
  public static ContactOperations updateExistingContact(Context context,long rawContactId,BatchOperation batchOperations){
    return new ContactOperations(context,rawContactId,batchOperations);
  }

}
