////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.miso;

import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import org.json.JSONObject;


/***********************************************************************************************
Details about a single Miso friend
***********************************************************************************************/
public class MisoFriend {

  /** The Miso user ID **/
  long userID = -1; // Custom Column "UserIdData"
  /** The users real name **/
  String realName = null;
  /** The users user name **/
  String username = null;
  /** The URL to the photo of this user **/
  URL photo = null;
  /** The Twitter account of this user **/
  String twitter = null;
  /** The Facebook account of this user **/
  String facebook = null;
  /** The tagline of the user **/
  String tagline = null;
  /** The website of the user **/
  String website = null;
  /** The last show this user checked-in to **/
  String checkin = null;
  /** The last time this user checked-in to a show **/
  long checkinTime = 0;
  /** The shoutout message that this user gave at the last check-in **/
  String shoutout = null;


  /***********************************************************************************************
  Constructor
  ***********************************************************************************************/
  public MisoFriend(){}


  /***********************************************************************************************
  Creates a new Miso friend from JSON data
  @param friend the JSON data for the friend
  ***********************************************************************************************/
  public MisoFriend(JSONObject friend){
    toComponent(friend);
  }


  /***********************************************************************************************
  Gets the last check-in location for this user
  @return the <i>checkin</i>
  ***********************************************************************************************/
  public String getCheckin(){
    return checkin;
  }


  /***********************************************************************************************
  Gets the last check-in time for this user
  @return the <i>checkinTime</i>
  ***********************************************************************************************/
  public long getCheckinTime(){
    return checkinTime;
  }


  /***********************************************************************************************
  Gets the Facebook account of this user
  @return the <i>facebook</i> account
  ***********************************************************************************************/
  public String getFacebook(){
    return facebook;
  }


  /***********************************************************************************************
  Gets the URL for the photo of this user
  @return the <i>photo</i> URL
  ***********************************************************************************************/
  public URL getPhoto(){
    return photo;
  }


  /***********************************************************************************************
  Gets the real name of this user
  @return the <i>realName</i>
  ***********************************************************************************************/
  public String getRealName(){
    return realName;
  }


  /***********************************************************************************************
  Gets the shoutout message for this user
  @return the <i>shoutout</i> message
  ***********************************************************************************************/
  public String getShoutout(){
    return shoutout;
  }


  /***********************************************************************************************
  Gets the tagline of this user
  @return the <i>tagline</i>
  ***********************************************************************************************/
  public String getTagline(){
    return tagline;
  }


  /***********************************************************************************************
  Gets the Twitter account of this user
  @return the <i>twitter</i> account
  ***********************************************************************************************/
  public String getTwitter(){
    return twitter;
  }


  /***********************************************************************************************
  Gets the user ID of this user
  @return the <i>userID</i>
  ***********************************************************************************************/
  public long getUserID(){
    return userID;
  }


  /***********************************************************************************************
  Gets the username of this user
  @return the <i>username</i>
  ***********************************************************************************************/
  public String getUsername(){
    return username;
  }


  /***********************************************************************************************
  Gets the website of this user
  @return the <i>website</i>
  ***********************************************************************************************/
  public String getWebsite(){
    return website;
  }


  /***********************************************************************************************
  Sets the last check-in location for this user
  @param checkin the last check-in location for this user
  ***********************************************************************************************/
  public void setCheckin(String checkin){
    if (checkin != null && checkin.equals("null")){
      checkin = null;
    }
    this.checkin = checkin;
  }


  /***********************************************************************************************
  Sets the last check-in time for this user
  @param checkinTime the last check-in time for this user
  ***********************************************************************************************/
  public void setCheckinTime(long checkinTime){
    this.checkinTime = checkinTime;
  }
  
  
  /***********************************************************************************************
  Sets the last check-in time for this user. Converts the String into a Long
  @param checkinTime the last check-in time for this user
  ***********************************************************************************************/
  public void setCheckinTime(String checkinTime){

    try {
      SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
      Date date = dateFormatter.parse(checkinTime);
      Calendar calendar = Calendar.getInstance();
      calendar.setTime(date);
      this.checkinTime = calendar.getTimeInMillis();
    }
    catch (Throwable t) {
      this.checkinTime = 0;
    }

  }


  /***********************************************************************************************
  Sets the Facebook account of this user
  @param facebook the Facebook account of this user
  ***********************************************************************************************/
  public void setFacebook(String facebook){
    if (facebook != null && facebook.equals("null")){
      facebook = null;
    }
    this.facebook = facebook;
  }


  /***********************************************************************************************
  Sets the URL to the photo of this user
  @param photo the URL to the photo of this user
  ***********************************************************************************************/
  public void setPhoto(URL photo){
    this.photo = photo;
  }


  /***********************************************************************************************
  Sets the real name of this user
  @param realName the real name of this user
  ***********************************************************************************************/
  public void setRealName(String realName){
    if (realName != null && realName.equals("null")){
      realName = null;
    }
    this.realName = realName;
  }


  /***********************************************************************************************
  Sets the shoutout message for this user
  @param shoutout the last shoutout message for this user
  ***********************************************************************************************/
  public void setShoutout(String shoutout){
    if (shoutout != null && shoutout.equals("null")){
      shoutout = null;
    }
    this.shoutout = shoutout;
  }


  /***********************************************************************************************
  Sets the tagline of this user
  @param tagline the tagline
  ***********************************************************************************************/
  public void setTagline(String tagline){
    if (tagline != null && tagline.equals("null")){
      tagline = null;
    }
    this.tagline = tagline;
  }


  /***********************************************************************************************
  Sets the Twitter account of this user
  @param twitter the Twitter account of this user
  ***********************************************************************************************/
  public void setTwitter(String twitter){
    if (twitter != null && twitter.equals("null")){
      twitter = null;
    }
    this.twitter = twitter;
  }


  /***********************************************************************************************
  Sets the User ID of this user
  @param userID the User ID of this user
  ***********************************************************************************************/
  public void setUserID(long userID){
    this.userID = userID;
  }


  /***********************************************************************************************
  Sets the username of this user
  @param username the username of this user
  ***********************************************************************************************/
  public void setUsername(String username){
    if (username != null && username.equals("null")){
      username = null;
    }
    this.username = username;
  }


  /***********************************************************************************************
  Sets the website of this user
  @param website the <i>website</i>
  ***********************************************************************************************/
  public void setWebsite(String website){
    if (website != null && website.equals("null")){
      website = null;
    }
    this.website = website;
  }


  /***********************************************************************************************
  Populates the MisoFriend from JSON data
  @param friend the JSON data for the friend
  ***********************************************************************************************/
  public void toComponent(JSONObject friend){
    try {
      setUserID(friend.optLong("id",-1));
      setRealName(friend.optString("full_name",""));
      setUsername(friend.optString("username",""));
      setWebsite(friend.optString("url",""));
      setTagline(friend.optString("tagline",""));

      String photoUrl = friend.optString("profile_image_url",null);
      if (photoUrl != null && !(photoUrl.equals("null"))){
        setPhoto(new URL(photoUrl));
      }
      
      JSONObject facebookWrapper = friend.optJSONObject("facebook");
      if (facebookWrapper != null){
        setFacebook(facebookWrapper.optString("id"));
      }

      JSONObject twitterWrapper = friend.optJSONObject("twitter");
      if (twitterWrapper != null){
        setTwitter(twitterWrapper.optString("id"));
      }

    }
    catch (Throwable t) {}

  }

}
