////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.miso;

import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.watto.program.android.sync.miso.authenticator.MisoAuthenticator;


/***********************************************************************************************
 Provides utility methods for communicating with the backend server
***********************************************************************************************/
public class MisoQuery {

  /***********************************************************************************************
  Fetches the list of friend data updates from the server
  @return list The list of updates received from the server.
  ***********************************************************************************************/
  public static MisoFriend[] getFriends(){
    try {

      if (!MisoAuthenticator.credentialsKnown()) {
        // The authentication is invalid, so return nothing
        return new MisoFriend[0];
      }

      /*
      String friendsURL = MisoAuthenticator.signRequest("http://gomiso.com/api/oauth/v1/users/follows.json");
      
      
      // Sign the request with the access token
      //friendsURL = MisoAuthenticator.signRequest(friendsURL);
      HttpGet request = new HttpGet(friendsURL);

      // Send the request
      DefaultHttpClient client = new DefaultHttpClient();
      HttpResponse response = client.execute(request);
      */

      /*
      // Build a HttpClient that ignores SSL errors
      SchemeRegistry schemeRegistry = new SchemeRegistry();
      schemeRegistry.register(new Scheme("http",PlainSocketFactory.getSocketFactory(),80));
      schemeRegistry.register(new Scheme("https",new EasySSLSocketFactory(),443));

      HttpParams params = new BasicHttpParams();
      ClientConnectionManager clientConnectionManager = new ThreadSafeClientConnManager(params,schemeRegistry);

      DefaultHttpClient client = new DefaultHttpClient(clientConnectionManager,params);
      HttpGet get = new HttpGet(friendsURL);

      // Call the URL
      HttpResponse response = null;
      try {
        response = client.execute(get);
      }
      catch (ClientProtocolException e) {}
      catch (IOException e) {}
      */

      String responseMessage = queryServer("http://gomiso.com/api/oauth/v1/users/follows.json");

      /*
      // Get the response code
      if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
        // Success
        String responseMessage = EntityUtils.toString(response.getEntity());
        */
      if (responseMessage != null) {

        JSONArray friendArray = new JSONArray(responseMessage);

        int friendCount = friendArray.length();
        MisoFriend[] friends = new MisoFriend[friendCount];

        // Create the friends
        for (int i = 0;i < friendCount;i++) {
          friends[i] = new MisoFriend(friendArray.getJSONObject(i).getJSONObject("user"));
        }

        // get further details about each friend
        getFriendsDetails(friends);

        return friends;
      }

      return new MisoFriend[0];
    }
    catch (Throwable t) {
      // An error occurred in the response, or there were parsing errors
      return new MisoFriend[0];
    }
  }


  /***********************************************************************************************
  Fetches further details for a list of <code>MisoFriend</code>s from the server
  @param friends The friends being queried
  ***********************************************************************************************/
  public static void getFriendsDetails(MisoFriend[] friends){
    try {

      int friendCount = friends.length;
      for (int i = 0;i < friendCount;i++) {
        MisoFriend friend = friends[i];

        long userId = friend.getUserID();

        String responseMessage = queryServer("http://gomiso.com/api/oauth/v1/checkins.json?user_id=" + userId + "&count=1");

        if (responseMessage == null) {
          // Error in the request, so go to the next user
          continue;
        }

        try {
          JSONArray responseWrapper = new JSONArray(responseMessage);
          JSONObject checkinWrapper = responseWrapper.getJSONObject(0).getJSONObject("checkin");
          
          friend.setCheckin(checkinWrapper.optString("media_title",""));
          friend.setCheckinTime(checkinWrapper.optString("created_at",""));
          friend.setShoutout(checkinWrapper.optString("comment",""));
        }
        catch (JSONException e) {
          // Error converting the response to a JSON Object
        }

      }

    }
    catch (Throwable t) {
      // Error in the response - maybe the server is down or the response message has changed
    }
  }


  /***********************************************************************************************
  Signs a query, sends it to the server, receives a response and returns it 
  @param queryUrl the URL of the query you wish to perform
  @return the response as a <code>String</code>, or <b>null</b> if there was an error
  ***********************************************************************************************/
  public static String queryServer(String queryUrl){
    try {

      // Check that we have valid authentication
      if (!MisoAuthenticator.credentialsKnown()) {
        return null;
      }

      // Sign the request with the access token
      queryUrl = MisoAuthenticator.signRequest(queryUrl);
      HttpGet request = new HttpGet(queryUrl);

      // Send the request
      DefaultHttpClient client = new DefaultHttpClient();
      HttpResponse response = client.execute(request);

      // Get the response code
      if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
        // Success
        String responseMessage = EntityUtils.toString(response.getEntity());

        // return the response
        return responseMessage;
      }
      return null;
    }
    catch (Throwable t) {
      return null;
    }
  }

}
