////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.miso.authenticator;

import org.watto.android.Settings;
import org.watto.android.component.WSAndroidPopup;
import org.watto.android.event.WSViewClickableInterface;
import org.watto.android.event.listener.WSViewClickableListener;
import org.watto.program.android.sync.miso.R;
import android.accounts.AccountAuthenticatorActivity;
import android.accounts.AccountManager;
import android.app.AlarmManager;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;


/***********************************************************************************************
Activity which displays login screen to the user
***********************************************************************************************/
public class AuthenticatorActivity extends AccountAuthenticatorActivity implements WSViewClickableInterface {

  boolean addAccount = false;

  public int ACTIVITY_CALL_BROWSER = 2;


  /***********************************************************************************************
  Called when response is received from the server for authentication request. See
  onAuthenticationResult(). Sets the AccountAuthenticatorResult which is sent back to the caller.
  Also sets the authToken in AccountManager for this account.
  @param the confirmCredentials result.
  ***********************************************************************************************/
  public void createAccount(){

    AccountManager accountManager = AccountManager.get(this);
    //MisoAuthenticator.createAccount(accountManager);

    // Allow regular syncing for Android 2.2+
    int apiVersion = android.os.Build.VERSION.SDK_INT;
    if (apiVersion <= 7) {
      // <= Android 2.1
      AlarmManager alarmManager = (AlarmManager)this.getSystemService(Context.ALARM_SERVICE);
      MisoAuthenticator.createAccount(accountManager,alarmManager,getBaseContext());
    }
    else if (apiVersion >= 8) {
      // Android 2.2+
      MisoAuthenticator.createAccount(accountManager,getContentResolver());
    }

    Intent accountIntent = MisoAuthenticator.getAccountIntent(accountManager);
    setAccountAuthenticatorResult(accountIntent.getExtras());
    setResult(RESULT_OK,accountIntent);

    finish();
  }


  /***********************************************************************************************
  Processes events that are triggered when a sub-<code>Activity</code> is <code>finish()</code>ed
  @param requestCode the code of the <code>Activity</code> that has finished
  @param resultCode the result of the <code>Activity</code>
  @param data the data returned from the Activity
  ***********************************************************************************************/
  public void onActivityResult(int requestCode,int resultCode,Intent data){
    super.onActivityResult(requestCode,resultCode,data);

    if (requestCode == ACTIVITY_CALL_BROWSER) {
      // Show the Verifying and Syncing screen, so the user is kept informed
      setContentView(R.layout.verifying);

      // Open a new Thread to run the Callback processing (takes it out of the GUI thread)
      new Thread(new AuthenticatorCallbackThread(this,data)).start();
    }
  }


  /***********************************************************************************************
  Performs an action when a single-click event is triggered on a <code>View</code>
  @param source the <code>View</code> that triggered the event
  @return <b>true</b> if the event was handled by this class<br />
          <b>false</b> if the event wasn't handled by this class, and thus should be passed on to
          the parent class for handling.
  ***********************************************************************************************/
  public boolean onClick(View source){
    int id = source.getId();
    if (id == R.id.button_add_account) {
      // Open the URL for adding an account
      Intent callBrowser = MisoAuthenticator.getAuthenticationIntent(this);

      if (callBrowser == null) {
        // Internet Access Error
        WSAndroidPopup.showError(this,R.string.error_connection_failure);
        finish();
      }

      startActivityForResult(callBrowser,ACTIVITY_CALL_BROWSER);
    }
    else if (id == R.id.button_update_credentials) {
      // Open the URL for updating the credentials for an account
      Intent callBrowser = MisoAuthenticator.getAuthenticationIntent(this);

      if (callBrowser == null) {
        // Internet Access Error
        WSAndroidPopup.showError(this,R.string.error_connection_failure);
        finish();
      }

      startActivityForResult(callBrowser,ACTIVITY_CALL_BROWSER);
    }
    else {
      return false;
    }
    return true;
  }


  /***********************************************************************************************
  Creates the <code>Activity</code>
  ***********************************************************************************************/
  public void onCreate(Bundle bundle){
    super.onCreate(bundle);

    // Set the Settings Context
    Settings.setApplication(this.getBaseContext());

    addAccount = false;

    // Get the Intent that started this activity
    Intent intent = getIntent();

    // Get the action that is triggering this Activity
    String action = intent.getStringExtra("action");

    if (action.equals("addAccount")) {
      addAccount = true;

      // Load the Add Account information screen
      setContentView(R.layout.add_account);

      // Register click listeners
      WSViewClickableListener clickListener = new WSViewClickableListener(this);
      findViewById(R.id.button_add_account).setOnClickListener(clickListener);
    }
    else if (action.equals("confirmCredentials")) {
      Intent callBrowser = MisoAuthenticator.getAuthenticationIntent(this);

      if (callBrowser == null) {
        // Internet Access Error
        WSAndroidPopup.showError(this,R.string.error_connection_failure);
        finish();
      }

      startActivityForResult(callBrowser,ACTIVITY_CALL_BROWSER);
    }
    else if (action.equals("getAuthToken")) {
      Intent callBrowser = MisoAuthenticator.getAuthenticationIntent(this);

      if (callBrowser == null) {
        // Internet Access Error
        WSAndroidPopup.showError(this,R.string.error_connection_failure);
        finish();
      }

      startActivityForResult(callBrowser,ACTIVITY_CALL_BROWSER);
    }
    else if (action.equals("updateCredentials")) {
      // Load the Add Account information screen
      setContentView(R.layout.update_credentials);

      // Register click listeners
      WSViewClickableListener clickListener = new WSViewClickableListener(this);
      findViewById(R.id.button_update_credentials).setOnClickListener(clickListener);
    }

  }


  /***********************************************************************************************
  Resumes the <code>Activity</code> (captures the callback URL)
  ***********************************************************************************************/
  public void onNewIntent(Intent intent){

    if (intent == null) {
      // Hit the Back button while in the Webview. So, want to close the whole of the
      // "account generation" stuff and return to the "Accounts And Sync" menu
      finish();
      return;
    }

    super.onNewIntent(intent);

    Uri uri = intent.getData();
    if (uri != null) {

      // Get the code from the callback URL
      String authToken = uri.getQueryParameter("oauth_token");
      String verifier = uri.getQueryParameter("oauth_verifier");
      // request a new access token for this application
      MisoAuthenticator.retreiveAccessToken(authToken,verifier);

      if (addAccount) {
        // Create the new account
        createAccount();
      }
      else {
        // Just update the current account with the new credentials
        updateAccount();
      }

      finish();
    }

  }


  /***********************************************************************************************
  Called when response is received from the server for authentication request. See
  onAuthenticationResult(). Sets the AccountAuthenticatorResult which is sent back to the caller.
  Also sets the authToken in AccountManager for this account.
  @param the confirmCredentials result.
  ***********************************************************************************************/
  public void updateAccount(){
    AccountManager accountManager = AccountManager.get(this);
    MisoAuthenticator.updateAccount(accountManager);

    Intent accountIntent = MisoAuthenticator.getAccountIntent(accountManager);
    setAccountAuthenticatorResult(accountIntent.getExtras());
    setResult(RESULT_OK,accountIntent);

    finish();
  }

}
